---@diagnostic disable: missing-return

account = {}

--- Get the current account's name.
--- @return string
function account.name() end

--- Get the current account's email.
--- @return string
function account.email() end

--- Get the current account's RID.
--- @return number
function account.rid() end

--- Get stats for a specified type.
--- @param stat string The name of the stat to retrieve
--- @return stat_data
function account.stats(stat) end

--- Data type representing various statistics.
--- @class stat_data
--- @field str string A string value
--- @field int32 number A 32-bit integer value
--- @field int64 number A 64-bit integer value
--- @field uint8 number An 8-bit unsigned integer value
--- @field uint16 number A 16-bit unsigned integer value
--- @field uint32 number A 32-bit unsigned integer value
--- @field uint64 number A 64-bit unsigned integer value
--- @field float number A floating-point number value
--- @field bool boolean A boolean value


chat = {}

--- Sends a message to the target. Available variables in the message: `${ip}`, `${name}`
--- @param message string: The message to send.
--- @param options { show_locally: boolean, targets: player[] }
function chat.send(message, options) end


chat = {}

--- Request control of an entity. This function may require several tries to successfully take control.
--- @param entity number|string Entity identifier (either a number or a string)
--- @param take_owner boolean? Whether the script becomes the owner of the entity. Defaults to false.
function request.control(entity, take_owner) end

--- Request loading of a specific model into the game.
--- @param hash number|string Model hash or string identifier to be loaded.
function request.model(hash) end

--- Request loading of a specific weapon into the game.
--- @param hash number|string Weapon hash or string identifier to be loaded.
function request.weapon(hash) end

--- Request loading of a specific dictionary into the game.
--- @param name string Name of the dictionary to be loaded.
function request.dict(name) end

--- Request loading of a specific animation into the game.
--- @param name string Name of the animation to be loaded.
function request.anim(name) end

--- Request loading of a specific particle asset into the game.
--- @param asset string Name of the particle asset to be loaded.
function request.particle(asset) end


dirs = {}

--- List the file names in a directory.
--- @param path string: The directory path to list.
--- @param extension string: (optional) Filter files by this extension.
--- @return string[] --An array of file names.
function dirs.list(path, extension) end

--- Get the path of a directory by its index.
--- @param index Directory: The index of the directory.
--- @return string --The path of the directory.
function dirs.get(index) end


events = {}

---@class event
---@field enabled boolean --Indicates if the event handler is enabled or disabled.
---@field callback function --The callback function that is called when the event is triggered.

--- Subscribes to an event with the specified callback function.
-- TODO: REPLACE NAME WITH ENUM
--- @param name Event --The name of the event to subscribe to.
-- TODO: TYPE OF DATA
--- @param callback fun(data: any) --The callback function that will be called when the event occurs.
--- @return number --Returns a handle to the subscribed event, which can be used to unsubscribe later.
function events.subscribe(name, callback) end

--- Unsubscribes from an event using the handle returned by subscribe.
--- @param handle number --The handle of the event to unsubscribe from.
function events.unsubscribe(handle) end

--- Retrieves an event handler by its handle.
--- @param handle number --The handle of the event to retrieve.
--- @return event --Returns the event object associated with the handle.
function events.get(handle) end


files = {}

---@class file
---@field good boolean @Whether the file is open
---@field text string @Read/write as text
---@field json nlohmann_json @Read/write as `nlohmann_json`

---Checks if the specified file exists in the directory.
---@param path string
---@return boolean
function files.exists(path) end

---Removes the specified file from the directory.
---@param path string
---@return boolean @Returns whether the file was successfully deleted.
function files.remove(path) end

---Opens a file for reading or writing, with options to create if not exists or append to it.
---@param path string
---@param options { create_if_not_exists: boolean, append: boolean }
---@return file
function files.open(path, options) end


game = {}

--- Triggers a script event
--- @param params table
--- @param bitset number
function game.script_event(params, bitset) end

--- Invites a player by RID to your session
--- @param rid number
function game.invite(rid) end

--- The pointer to the net object for the given id
--- @param id number
--- @return memory_address
function game.net_object_from_id(id) end

--- Send a friend request to a player by RID
--- @param rid number
function game.add_friend(rid) end

--- Whether the RID is your friend
--- @param rid number
--- @return bool
function game.is_rid_friend(rid) end

--- Joins a player by RID
--- @param rid number
--- @param spectate bool?
function game.join_player(rid, spectate) end

--- Triggers a transaction
--- @param category number
--- @param action number
--- @param destination number
--- @param items table<number[5]>
function game.basket_transaction(category, action, destination, items) end

--- Whether the transaction manager is busy
--- @return bool
function game.is_basket_busy() end

--- Get the pointer to the model's info
--- @param hash number
--- @return memory_address
function game.get_model_info(hash) end

--- Updates a networked entity's owner
--- @param handle number
--- @param new_owner player
--- @return bool
function game.update_entity_owner(handle, new_owner) end

--- Get the entity's owner
--- @param handle number
--- @return player
function game.get_entity_owner(handle) end

--- Get the address for an entity handle
--- @param handle number
--- @return memory_address
function game.get_entity_from_guid(handle) end

--- Executes a function in the given script's context
--- @param script string
--- @param callback fun()
function game.execute_as_script(script, callback) end

--- Starts a script for the session
--- @param id number
function game.start_script(id) end

--- Changes session
--- @param id number
function game.change_session(id) end

--- Gets the game state
--- @return GameState
function game.state() end

--- Gets the delta time
--- @return number
function game.delta() end

--- Gets the game resolution
--- @return vec2
function game.resolution() end

http = {}

--- Response type for HTTP requests
---@class response
---@field success boolean: Whether the request succeeded
---@field status number: The HTTP status code
---@field headers table<string, string>: The response headers
response = {}

--- Returns the response body in binary format
---@function response:body
---@return string: The response body in binary format
function response:body() end

--- Returns the response body in text format
---@function response:text
---@return string: The response body in text format
function response:text() end

--- Returns the response body as JSON
---@function response:json
---@return table: The response body parsed as JSON
function response:json() end

--- Performs an HTTP request
---@function fetch
---@param url string
---@param options? { method: string, headers: table<string, string> }
---@param body? string
---@return response
function http.fetch(url, options, body) end

--- Performs an HTTP request asynchronously
---@function fetch_async
---@param url string
---@param options? { method: string, headers: table<string, string> }
---@param callback fun(res: response) --Callback function that receives the response
---@return nil
function http.fetch_async(url, options, callback) end


input = {}

---@class input_pad_state
---@field pressed boolean @Whether the key is pressed
---@field just_pressed boolean @Whether the key was just pressed
---@field just_released boolean @Whether the key was just released

---@class input_mouse_state
---@field double_clicked boolean @Whether the key just double pressed
---@field pressed boolean @Whether the key is held down
---@field just_pressed boolean @Whether the key was just pressed
---@field just_released boolean @Whether the key was just released
---@field name string @Name of the key

---@class input_keyboard_state
---@field pressed boolean @Whether the key is held down
---@field just_pressed boolean @Whether the key was just pressed
---@field just_released boolean @Whether the key was just released
---@field name string @Name of the key

---@class vec2
---@field x number
---@field y number

---@overload fun(key: number):input_pad_state
---@param key number
---@return input_pad_state
function input.pad(key) end

---@overload fun(vk: number):input_keyboard_state
---@param vk number
---@return input_keyboard_state
function input.key(vk) end

---@param key number
---@return input_mouse_state
function input.mouse(key) end

---@return input_keyboard_state
function input.last_key() end

---@return vec2
function input.mouse_position() end

---@overload fun(xy: vec2, wh: vec2): boolean
---@param xy vec2
---@param wh vec2
---@return boolean
function input.is_mouse_in_bounds(xy, wh) end

---@overload fun(toggled: boolean):nil
---@param toggled boolean
function input.cursor(toggled) end


json = {}

--- Converts a `nlohmann_json` object to a Lua table.
---@param json nlohmann_json @The JSON object to convert.
---@return table @The resulting Lua table.
function json.to_table(json) end

--- Converts a Lua table to a `nlohmann_json` object.
---@param tbl table @The Lua table to convert.
---@return nlohmann_json @The resulting JSON object.
function json.from_table(tbl) end

---@class nlohmann_json
local nlohmann_json = {}

--- Stringifies the JSON object.
---@param indent number? @Optional indentation for the resulting string.
---@return string @The JSON object as a string.
function nlohmann_json:dump(indent) end


--- Creates a new vec2 instance with coordinates (0, 0).
--- @return vec2
--- @overload fun(xy: number): vec2
--- @overload fun(x: number, y: number): vec2
function vec2() end

do
  --- @class vec2
  --- @field x number
  --- @field y number
  local vec2 = {}

  --- Normalizes the vec2 instance.
  --- @return vec2
  function vec2:normalize() end

  --- Computes the dot product of this vector with another vec2.
  --- @param other vec2
  --- @return number
  function vec2:dot(other) end
end

--- Creates a new vec3 instance with coordinates (0, 0, 0).
--- @return vec3
--- @overload fun(xyz: number): vec3
--- @overload fun(x: number, y: number, z: number): vec3
function vec3() end

do
  --- @class vec3
  --- @field x number
  --- @field y number
  --- @field z number
  local vec3 = {}

  --- Normalizes the vec3 instance.
  --- @return vec3
  function vec3:normalize() end

  --- Computes the dot product of this vector with another vec3.
  --- @param other vec3
  --- @return number
  function vec3:dot(other) end

  --- Calculates the 3D distance between this vector and another vec3.
  --- @param other vec3
  --- @return number
  function vec3:distance3d(other) end
end

--- Creates a new scr_vec3 instance with coordinates (0, 0, 0).
--- @return scr_vec3
--- @overload fun(xyz: number): scr_vec3
--- @overload fun(x: number, y: number, z: number): scr_vec3
function scr_vec3() end

do
  --- @class scr_vec3
  --- @field x number
  --- @field y number
  --- @field z number
  local scr_vec3 = {}

  --- Normalizes the scr_vec3 instance.
  --- @return scr_vec3
  function scr_vec3:normalize() end

  --- Computes the dot product of this vector with another scr_vec3.
  --- @param other scr_vec3
  --- @return number
  function scr_vec3:dot(other) end

  --- Calculates the 3D distance between this vector and another scr_vec3.
  --- @param other scr_vec3
  --- @return number
  function scr_vec3:distance3d(other) end
end


memory = {}

--- Returns the base address of the game.
--- @return number @The base address.
function memory.base() end

--- Returns the size of the game.
--- @return number @The size.
function memory.size() end

--- Allocates memory of the given size.
--- @param size number @The size of memory to allocate.
--- @return allocated_memory @The allocated memory block.
function memory.alloc(size) end

--- Frees allocated memory.
--- @param mem allocated_memory @The memory block to free.
function memory.free(mem) end

--- Scans memory for a signature.
--- @param signature string @The signature to look for.
--- @param module string? @The module within which to search. Optional.
--- @return memory_address @The address where the signature is found.
function memory.scan(signature, module) end

--- Calls a function by address.
--- @param address number @The memory address of the function.
--- @vararg any @The parameters to pass to the function.
--- @return memory_address @The result of the function call.
function memory.call(address, ...) end

--- Type representing a block of allocated memory.
--- @class allocated_memory
--- @operator call:memory_address @Access memory address.

do
  --- Type representing a memory address.
  --- @class memory_address
  --- @field value number @The actual memory address.
  --- @field int8 number @Read as int8.
  --- @field int16 number @Read as int16.
  --- @field int32 number @Read as int32.
  --- @field int64 number @Read as int64.
  --- @field uint8 number @Read as uint8.
  --- @field uint16 number @Read as uint16.
  --- @field uint32 number @Read as uint32.
  --- @field uint64 number @Read as uint64.
  --- @field float number @Read as float.
  --- @field double number @Read as double.
  --- @field bool boolean @Read as boolean.
  --- @field str string @Read as string.
  --- @field vec2 vec2 @Read as vec2.
  --- @field vec3 vec3 @Read as vec3.
  --- @field scr_vec3 scr_vec3 @Read as scr_vec3.
  --- @field as_int8 number @Cast to int8.
  --- @field as_int16 number @Cast to int16.
  --- @field as_int32 number @Cast to int32.
  --- @field as_int64 number @Cast to int64.
  --- @field as_uint8 number @Cast to uint8.
  --- @field as_uint16 number @Cast to uint16.
  --- @field as_uint32 number @Cast to uint32.
  --- @field as_uint64 number @Cast to uint64.
  --- @field as_float number @Cast to float.
  --- @field as_double number @Cast to double.
  --- @field as_bool boolean @Cast to boolean.
  --- @field as_str string @Cast to string.
  --- @field as_vec2 vec2 @Cast to vec2.
  --- @field as_vec3 vec3 @Cast to vec3.
  --- @field as_scr_vec3 scr_vec3 @Cast to scr_vec3.
  local memory_address = {}

  --- Adds an offset to the address and returns the new `memory_address`.
  --- @param offset number @The offset to add.
  --- @return memory_address @The new memory address.
  function memory_address:add(offset) end

  --- Subtracts an offset from the address and returns the new `memory_address`.
  --- @param offset number @The offset to subtract.
  --- @return memory_address @The new memory address.
  function memory_address:sub(offset) end

  --- Writes `count` `NOP` instructions at the address.
  --- @param count number @The number of NOP instructions to write.
  function memory_address:nop(count) end

  --- Extracts data from an instruction at the address using `opsize` and `instsize`.
  --- @param opsize number @The operand size.
  --- @param instsize number @The instruction size.
  --- @return memory_address @The extracted data.
  function memory_address:rip(opsize, instsize) end
end

--- Constructs a `memory_address`.
--- @param address number @The address to wrap.
--- @return memory_address @The wrapped address.
function memory_address(address) end


menu = {}

--- Root of your scripts dedicated submenu
---@return menu_skeleton
function menu.root() end

--- Root of your scripts dedicated submenu inside of `Network` > `Players`
---@return menu_skeleton
function menu.player_root() end

--- Represents a submenu
---@class menu_skeleton
---@field options menu_class_id[]
local menu_skeleton = {}

--- Resizes the submenu
---@param size number
---@param callback fun(skeleton: menu_skeleton)
function menu_skeleton:resize(size, callback) end

--- Called when the submenu is entered
---@param callback fun(skeleton: menu_skeleton)
function menu_skeleton:on_enter(callback) end

--- Called when the submenu is left
---@param callback fun(skeleton: menu_skeleton)
function menu_skeleton:on_leave(callback) end

--- Creates a new submenu
---@param name string
---@param tooltip string?
---@return menu_skeleton
function menu_skeleton:submenu(name, tooltip) end

--- Creates a new button
---@param name string
---@return menu_button
function menu_skeleton:button(name) end

--- Creates a new breaker
---@param name string
---@return menu_breaker
function menu_skeleton:breaker(name) end

--- Creates a new color
---@param name string
---@return menu_color
function menu_skeleton:color(name) end

--- Creates a new toggle
---@param name string
---@return menu_toggle
function menu_skeleton:toggle(name) end

--- Creates a new label
---@param name string
---@return menu_label
function menu_skeleton:label(name) end

--- Creates a new textbox
---@param name string
---@return menu_textbox
function menu_skeleton:textbox(name) end

--- Creates a new number
---@param name string
---@param type NumberType
---@param action OptScroll
---@return menu_number
function menu_skeleton:number(name, type, action) end

--- Creates a new combo
---@param name string
---@param type ComboType
---@param list table<table<string, any>[]>
---@param action OptScroll?
---@param multi boolean?
---@return menu_combo
function menu_skeleton:combo(name, type, list, action, multi) end

--- Represents a menu option
---@class menu_class_id
---@field id OptClassId
local menu_class_id = {}

---@return menu_base
function menu_class_id:as_base() end

---@return menu_button
function menu_class_id:as_button() end

---@return menu_breaker
function menu_class_id:as_breaker() end

---@return menu_color
function menu_class_id:as_color() end

---@return menu_toggle
function menu_class_id:as_toggle() end

---@return menu_label
function menu_class_id:as_label() end

---@return menu_textbox
function menu_class_id:as_textbox() end

---@param type NumberType
---@return menu_number
function menu_class_id:as_number(type) end

---@param type ComboType
---@return menu_combo
function menu_class_id:as_combo(type) end

---@return menu_skeleton
function menu_class_id:as_skeleton() end

---@class menu_base
---@field name string
---@field type OptType
---@field sub_type OptSubType

---@class menu_breaker : menu_base
local menu_breaker = {}

---@class menu_button : menu_base
local menu_button = {}

---@param flags OptFlag
---@return menu_button
function menu_button:has_flags(flags) end

---@param tooltip string
---@return menu_button
function menu_button:tooltip(tooltip) end

---@param callback fun(button: menu_button)
---@return menu_button
function menu_button:on_click(callback) end

---@param callback fun(button: menu_button)
---@return menu_button
function menu_button:on_enter(callback) end

---@param callback fun(button: menu_button)
---@return menu_button
function menu_button:on_leave(callback) end

---@class menu_color : menu_base
---@field value color
local menu_color = {}

---@param flags OptFlag
---@return menu_color
function menu_color:has_flags(flags) end

---@param tooltip string
---@return menu_color
function menu_color:tooltip(tooltip) end

---@param callback fun(color: menu_color)
---@return menu_color
function menu_color:on_click(callback) end

---@return menu_color
function menu_color:no_alpha() end

---@class menu_toggle : menu_base
---@field value boolean
local menu_toggle = {}

---@param key player
---@return toggle_player_lua_state
function menu_toggle:__index(key) end

---@param flags OptFlag
---@return menu_toggle
function menu_toggle:has_flags(flags) end

---@param tooltip string
---@return menu_toggle
function menu_toggle:tooltip(tooltip) end

---@param callback fun(toggle: menu_toggle)
---@return menu_toggle
function menu_toggle:on_click(callback) end

---@param callback fun(toggle: menu_toggle)
---@return menu_toggle
function menu_toggle:on_enter(callback) end

---@param callback fun(toggle: menu_toggle)
---@return menu_toggle
function menu_toggle:on_leave(callback) end

---@class toggle_player_lua_state
---@field value boolean

---@class menu_label : menu_base
---@field value string
local menu_label = {}

---@param flags OptFlag
---@return menu_label
function menu_label:has_flags(flags) end

---@param tooltip string
---@return menu_label
function menu_label:tooltip(tooltip) end

---@param callback fun(label: menu_label)
---@return menu_label
function menu_label:on_click(callback) end

---@return menu_label
function menu_label:disable_colon() end

---@class menu_textbox : menu_base
---@field value string
local menu_textbox = {}

---@param flags OptFlag
---@return menu_textbox
function menu_textbox:has_flags(flags) end

---@param tooltip string
---@return menu_textbox
function menu_textbox:tooltip(tooltip) end

---@param callback fun(textbox: menu_textbox, text: string)
---@return menu_textbox
function menu_textbox:on_submit(callback) end

---@param callback fun(textbox: menu_textbox)
---@return menu_textbox
function menu_textbox:on_enter(callback) end

---@param callback fun(textbox: menu_textbox)
---@return menu_textbox
function menu_textbox:on_leave(callback) end

---@param text string
---@return menu_textbox
function menu_textbox:default_value(text) end

---@param max number
---@return menu_textbox
function menu_textbox:max_chars(max) end

---@class menu_combo : menu_base
---@field value number
---@field toggle boolean
---@field multi menu_combo_multi_state
---@field list menu_combo_list
local menu_combo = {}

---@param key player
---@return combo_player_lua_state
function menu_combo:__index(key) end

---@param flags OptFlag
---@return menu_combo
function menu_combo:has_flags(flags) end

---@param tooltip string
---@return menu_combo
function menu_combo:tooltip(tooltip) end

---@param callback fun(combo: menu_combo)
---@return menu_combo
function menu_combo:on_click(callback) end

---@param callback fun(combo: menu_combo)
---@return menu_combo
function menu_combo:on_enter(callback) end

---@param callback fun(combo: menu_combo)
---@return menu_combo
function menu_combo:on_leave(callback) end

---@class combo_player_lua_state
---@field value number
---@field toggle boolean
---@field multi menu_combo_multi_state

---@class menu_combo_multi_state
---@field value string
---@field toggle boolean
---@field name string
---@field type OptType
---@field sub_type OptSubType
local menu_combo_multi_state = {}

---@param index number
---@return boolean
function menu_combo_multi_state:is_set(index) end

---@param index number
---@param value boolean
function menu_combo_multi_state:set(index, value) end

---@param index number
---@return boolean
function menu_combo_multi_state:is_disabled(index) end

---@param index number
---@param value boolean
function menu_combo_multi_state:disable(index, value) end

---@return number
function menu_combo_multi_state:count() end

---@param from number
---@param to number
---@return boolean
function menu_combo_multi_state:is_active_ranged(from, to) end

---@class menu_number : menu_base
---@field value number
---@field toggle boolean
local menu_number = {}

---@param key player
---@return number_player_lua_state
function menu_number:__index(key) end

---@param flags OptFlag
---@return menu_number
function menu_number:has_flags(flags) end

---@param tooltip string
---@return menu_number
function menu_number:tooltip(tooltip) end

---@param callback fun(number: menu_number)
---@return menu_number
function menu_number:on_click(callback) end

---@param callback fun(number: menu_number)
---@return menu_number
function menu_number:on_enter(callback) end

---@param callback fun(number: menu_number)
---@return menu_number
function menu_number:on_leave(callback) end

---@param step number
---@return menu_number
function menu_number:step(step) end

---@param fmt string
---@param min number
---@param max number
---@return menu_number
function menu_number:format(fmt, min, max) end

---@class number_player_lua_state
---@field value number
---@field toggle boolean @The toggle value if this option type is `OptScroll.TOGGLE`

---@class menu_combo_list
---@field name string
---@field value any
local menu_combo_list = {}

---@param index number
---@return menu_combo_list
function menu_combo_list:at(index) end


models = {}

---@class model_list
---@field name string The name of the model
---@field hash number The unique identifier hash for the model

--- Fetches the list of ped models used in Lexis.
--- @return model_list[] @List of ped models
function models.ped()
end

--- Fetches the list of object models used in Lexis.
--- @return model_list[] @List of object models
function models.object()
end

--- Fetches the list of vehicle models used in Lexis.
--- @return model_list[] @List of vehicle models
function models.vehicle()
end

--- Fetches the list of weapon models used in Lexis.
--- @return model_list[] @List of weapon models
function models.weapon()
end


invoker = {}

--- A type representing the return from a native call.
--- @class native_return
--- @field int number The integer value.
--- @field int64 number The integer value.
--- @field float number The float value.
--- @field bool boolean The boolean value.
--- @field str string The string value.
--- @field address memory_address The memory address.
--- @field vec3 scr_vec3 A list of three floats.

do
  --- Pointer to an integer.
  --- @class ptr_int
  --- @field value number The integer value.
  local ptr_int = {}

  function ptr_int:from_address(address) end -- Sets the data from a memory address.

  function ptr_int:address() end             -- Gets the value as a memory address.

  function ptr_int:free() end                -- Deletes the memory allocated.
end

do
  --- Pointer to a float.
  --- @class ptr_float
  --- @field value number The float value.
  local ptr_float = {}
  function ptr_float:from_address(address) end -- Sets the data from a memory address.

  function ptr_float:address() end             -- Gets the value as a memory address.

  function ptr_float:free() end                -- Deletes the memory allocated.
end

do
  --- Pointer to a boolean.
  --- @class ptr_bool
  --- @field value boolean The boolean value.
  local ptr_bool = {}
  function ptr_bool:from_address(address) end -- Sets the data from a memory address.

  function ptr_bool:address() end             -- Gets the value as a memory address.

  function ptr_bool:free() end                -- Deletes the memory allocated.
end

do
  --- Pointer to a script array.
  --- @class ptr_scr_array
  --- Can be used with `[]` operator.
  local ptr_scr_array = {}
  function ptr_scr_array:address() end -- Gets the value as a memory address.

  function ptr_scr_array:free() end    -- Deletes the memory allocated.
end

--- Invokes a native function.
--- @param hash number The hash of the native to invoke.
--- @vararg any Arguments to pass to the native.
--- @return native_return
function invoker.call(hash, ...) end

--- Creates a pointer for int values to be used with native calls.
--- @param count number? Optional count for array allocation.
--- @return ptr_int
function ptr_int(count) end

--- Creates a pointer for boolean values to be used with native calls.
--- @param count number? Optional count for array allocation.
--- @return ptr_bool
function ptr_bool(count) end

--- Creates a pointer for float values to be used with native calls.
--- @param count number? Optional count for array allocation.
--- @return ptr_float
function ptr_float(count) end

--- Creates a pointer for script array to be used with native calls.
--- @param count number The number of elements in the array.
--- @return ptr_scr_array
function ptr_scr_array(count) end


notify = {}

--- Clears the notification queue.
function notify.clear() end

--- Pushes a notification to the queue.
--- @param title string Title of the notification
--- @param desc string Description of the notification
--- @param options { time: number, icon: Notify }? Options for the notification
function notify.push(title, desc, options) end


players = {}

---@class player_geo
---@field valid boolean
---@field country string
---@field country_code string
---@field region string
---@field city string
---@field zip string
---@field timezone string
---@field isp string
---@field org string

---@class player
---@field name string
---@field relay_ipv4_formatted string
---@field ipv4_formatted string
---@field id number
---@field ped number
---@field entity number
---@field weapon_entity number
---@field vehicle number
---@field weapon number
---@field model number
---@field blacklist number
---@field rank number
---@field cash number
---@field bank number
---@field wallet number
---@field kd number
---@field heading number
---@field speed number
---@field bounty number
---@field playtime number
---@field rid number
---@field token number
---@field nat number
---@field relay_ipv4 number
---@field ipv4 number
---@field ping number
---@field exists boolean
---@field connected boolean
---@field alive boolean
---@field in_vehicle boolean
---@field has_weapon_in_hand boolean
---@field aiming boolean
---@field shooting boolean
---@field driver boolean
---@field friend boolean
---@field is_modder boolean
---@field p2p boolean
---@field coords scr_vec3
---@field rotation scr_vec3
---@field waypoint vec2
---@field ptr_ped memory_address
---@field ptr_vehicle memory_address
---@field ptr_net_player memory_address
---@field geo player_geo
local player = {}

---@alias scr_vec3 table<number, number, number>
---@alias vec2 table<number, number>
---@alias memory_address lightuserdata

---Returns your player
---@return player
function players.me() end

---Returns the selected player if player submenu is not the session
---@return player
function players.selected() end

---Returns the session host
---@return player
function players.host() end

---Returns the player by their index
---@param index number
---@return player
function players.get(index) end

---Returns the script host
---@param script string?
---@return player
function players.scr_host(script) end

---List of players
---@param bypass_excludes boolean?
---@return player[]
function players.list(bypass_excludes) end

---Whether the active player submenu is the session
---@return boolean
function players.is_session() end

---Blocks this net object type
---@param type NetBlock
---@param obj_type NetObj
---@param block boolean
function player:block(type, obj_type, block) end

---Increments a stat by the value
---@param hash number
---@param value number
function player:increment_stat(hash, value) end

---Force syncs a data node
---@param net_obj memory_address
---@param type NetObj
---@param node DataNodes
function player:sync_node(net_obj, type, node) end

---Teleports the player
---@param coords scr_vec3
function player:teleport(coords) end

--- Triggers a script event
--- @param params table
function player:script_event(params) end

pools = {}

--- Returns a list of peds in the game.
--- Each entry in the list is a tuple consisting of a handle and a memory address.
---@return table<number, memory_address>[] @List of peds as (handle, address)
function pools.ped() end

--- Returns a list of objects in the game.
--- Each entry in the list is a tuple consisting of a handle and a memory address.
---@return table<number, memory_address>[] @List of objects as (handle, address)
function pools.object() end

--- Returns a list of pickups in the game.
--- Each entry in the list is a tuple consisting of a handle and a memory address.
---@return table<number, memory_address>[] @List of pickups as (handle, address)
function pools.pickup() end

--- Returns a list of vehicles in the game.
--- Each entry in the list is a tuple consisting of a handle and a memory address.
---@return table<number, memory_address>[] @List of vehicles as (handle, address)
function pools.vehicle() end

--- Returns a list of weapons in the game.
--- Each entry in the list is a memory address.
---@return table<memory_address>[] @List of weapons as memory address
function pools.weapon() end


---Create a `regex` object
---@param pattern string The regex pattern to use
---@return regex The regex object
function regex(pattern) end

---@class regex
local regex_class = {}

---Finds all matches in the given `subject` string
---@param subject string The string to search in
---@return string[][] A table of matches, each match is a table of strings
function regex_class:findall(subject) end


gui = {}

--- Pushes a clipping rectangle defined by position `xy` and size `wh`.
---@param xy vec2 @Position of the clipping rectangle
---@param wh vec2 @Width and height of the clipping rectangle
function gui.push_clip(xy, wh) end

--- Pops the topmost clipping rectangle from the clipping stack.
function gui.pop_clip() end

--- Creates a text object with the specified content.
---@param text string @The text content
---@return text @The created text object
function gui.text(text) end

--- Creates a rectangle object with the specified position and size.
---@param xy vec2 @Position of the rectangle
---@param wh vec2 @Width and height of the rectangle
---@return rect @The created rectangle object
function gui.rect(xy, wh) end

--- Returns the size of the given text, optionally overridden by scale and wrapping width.
---@param text string @The text content
---@param override_scale number? @Override scale factor
---@param wrap number? @Wrapping width
---@return vec2 @The calculated size
function gui.get_size(text, override_scale, wrap) end

--- Renders an image at the specified position and size.
---@param img texture_data @The image data
---@param xy vec2 @Position of the image
---@param wh vec2 @Width and height of the image
---@param color color? @Color tint
function gui.image(img, xy, wh, color) end

--- Renders a rounded image at the specified position and size, with optional rounding amount and flags.
---@param img texture_data @The image data
---@param xy vec2 @Position of the image
---@param wh vec2 @Width and height of the image
---@param color color? @Color tint
---@param rounding number? @Rounding amount
---@param flags Rounding? @Rounding flags
function gui.rounded_image(img, xy, wh, color, rounding, flags) end

--- Loads an image from a file path.
---@param path string @Path to the image file
---@return texture_data @Loaded image data
function gui.load_image(path) end

--- Creates a keyframe animator
--- @return keyframe_animator
function gui.animator() end

--- Keyframes created with the keyframe animator
--- @class keyframe
--- @field position vec2
--- @field opacity number
--- @field scale number

do
  --- Represents the keyframe animator
  --- @class keyframe_animator
  --- @field current_frame keyframe @Current frame
  local keyframe_animator = {}

  --- @param percentage number @Percentage for the frame
  --- @return keyframe
  function keyframe_animator:frame(percentage) end

  --- @param delta number @Delta time, the current delta can be retrieved from game.delta()
  --- @return keyframe
  function keyframe_animator:update(delta) end

  --- @param update_time number @The speed in which the frames are updated
  --- @param delay number @The starting delay for this animator
  --- @return nil
  function keyframe_animator:init(update_time, delay) end

  --- @return boolean
  function keyframe_animator:valid() end

  --- @return nil
  function keyframe_animator:reset() end
end

--- Represents a color with RGB and optional alpha values.
---@class color
---@field r number @Red component (0 to 255)
---@field g number @Green component (0 to 255)
---@field b number @Blue component (0 to 255)
---@field rf number @Red component as a float (0.0 to 1.0)
---@field gf number @Green component as a float (0.0 to 1.0)
---@field bf number @Blue component as a float (0.0 to 1.0)

--- Constructs a color from hexadecimal or RGBA values.
---@overload fun(hex:number):color
---@param r number @Red component
---@param g number @Green component
---@param b number @Blue component
---@param a number? @Alpha component
---@return color @The constructed color
function color(r, g, b, a) end

--- Represents texture data with scale, name, and path identifiers.
---@class texture_data
---@field scale vec2 @The scale factor of the texture
---@field name string @The name of the texture
---@field path string @The path to the texture file
---@field crc number @The CRC checksum of the texture

do
  --- Represents a rectangle which can be manipulated with additional drawing parameters.
  ---@class rect
  local rect = {}

  ---@return rect
  function rect:line() end

  ---@return rect
  function rect:filled() end

  ---@param flags Rounding
  ---@return rect
  function rect:flags(flags) end

  ---@param col color
  ---@return rect
  function rect:color(col) end

  ---@param col color
  ---@return rect
  function rect:outline(col) end

  ---@param thickness number
  ---@return rect
  function rect:thickness(thickness) end

  ---@param rounding number
  ---@return rect
  function rect:rounding(rounding) end

  ---@param upr_left color
  ---@param upr_right color
  ---@param bot_right color
  ---@param bot_left color
  ---@return rect
  function rect:multicolors(upr_left, upr_right, bot_right, bot_left) end

  ---@return nil
  function rect:draw() end
end

do
  --- Represents a text object with formatting and drawing capabilities.
  ---@class text
  local text = {}

  ---@param pos vec2
  ---@return text
  function text:position(pos) end

  ---@param col color
  ---@return text
  function text:color(col) end

  ---@param justify Justify
  ---@return text
  function text:justify(justify) end

  ---@param w number
  ---@return text
  function text:wrap(w) end

  ---@return text
  function text:shadow() end

  ---@param xy vec2
  ---@param wh vec2
  ---@return text
  function text:clip(xy, wh) end

  ---@param t number
  ---@return text
  function text:outline_thickness(t) end

  ---@param c color
  ---@return text
  function text:outline(c) end

  ---@param r number
  ---@return text
  function text:override_scale(r) end

  ---@param col color
  ---@return text
  function text:box(col) end

  ---@return text
  function text:box_rounded() end

  ---@param r number
  ---@return text
  function text:box_rounding(r) end

  ---@param flags Rounding
  ---@return text
  function text:box_flags(flags) end

  ---@param padding vec2
  ---@return text
  function text:box_padding(padding) end

  ---@param col color
  ---@return text
  function text:rotate(angle) end

  ---@return nil
  function text:draw() end
end


scapi = {}

--- Represents a Social Club account with basic details.
--- @class scapi_rockstar_account
--- @field name string
--- @field rid number
--- @field success boolean

--- Represents pagination details for a query.
--- @class scapi_result_info
--- @field total number Total number of results
--- @field next_page number Next page number
--- @field query string Query that was made

--- Represents a paginated list of Social Club accounts.
--- @class scapi_account_pagination
--- @field accounts scapi_rockstar_account[] List of accounts
--- @field paged scapi_result_info Pagination details
--- @field success boolean Indicates if the query was successful

--- Queries a Social Club account by RID.
--- @param rid number The RID to query.
--- @return scapi_rockstar_account --The account details.
function scapi.query(rid) end

--- Queries a Social Club account by name, with optional pagination.
--- @param name string The account name to query.
--- @param page number? Optional page number for pagination.
--- @return scapi_account_pagination --The paginated account details.
function scapi.query(name, page) end

--- Asynchronously queries a Social Club account by RID.
--- @param rid number The RID to query.
--- @param callback fun(account: scapi_rockstar_account) Callback function with the account details.
--- @return nil
function scapi.query_async(rid, callback) end

--- Asynchronously queries a Social Club account by name, with optional pagination.
--- @param name string The account name to query.
--- @param callback fun(accounts: scapi_account_pagination) Callback function with the paginated account details.
--- @param page number? Optional page number for pagination.
--- @return nil
function scapi.query_async(name, callback, page) end


script = {}

---@class script_global
---@field public str string
---@field public vec3 scr_vec3
---@field public vec2 vec2
---@field public int8 number
---@field public int16 number
---@field public int32 number
---@field public int64 number
---@field public uint8 number
---@field public uint16 number
---@field public uint32 number
---@field public uint64 number
---@field public double number
---@field public float number
---@field public bool boolean
local script_global = {}

---@class script_local
---@field public str string
---@field public vec3 scr_vec3
---@field public vec2 vec2
---@field public int8 number
---@field public int16 number
---@field public int32 number
---@field public int64 number
---@field public uint8 number
---@field public uint16 number
---@field public uint32 number
---@field public uint64 number
---@field public double number
---@field public float number
---@field public bool boolean
local script_local = {}

---@class script_tunable
---@field public str string
---@field public vec3 scr_vec3
---@field public vec2 vec2
---@field public int8 number
---@field public int16 number
---@field public int32 number
---@field public int64 number
---@field public uint8 number
---@field public uint16 number
---@field public uint32 number
---@field public uint64 number
---@field public double number
---@field public float number
---@field public bool boolean
local script_tunable = {}

---@param script string
---@return boolean
function script.is_running(script) end

---@param base number
---@return script_global
function script.globals(base) end

---@param hash number
---@param base number
---@return script_local
function script.locals(hash, base) end

---@param hash number
---@return script_tunable
function script.tunables(hash) end

---@overload fun(index:number, size:number):script_global
---@overload fun(index:number):script_global
---@param index number
---@param size? number
---@return script_global
function script_global:at(index, size) end

---@overload fun(index:number, size:number):script_local
---@overload fun(index:number):script_local
---@param index number
---@param size? number
---@return script_local
function script_local:at(index, size) end


spawn = {}

--- Spawns a vehicle at the specified coordinates.
---@param model number|string The model hash or name of the vehicle to spawn.
---@param coords scr_vec3 The coordinates where the vehicle will spawn.
---@param heading number The heading of the vehicle.
---@param register_as_network_object boolean? If true, registers the vehicle as a network object.
---@param script_host_object boolean? If true, the vehicle is hosted by the script.
---@param ignore_ground_check boolean? If true, ignores ground collision checks when spawning.
---@return number The handle to the spawned vehicle.
function spawn.vehicle(model, coords, heading, register_as_network_object, script_host_object, ignore_ground_check) end

--- Spawns a pedestrian at the specified coordinates.
---@param model number|string The model hash or name of the pedestrian to spawn.
---@param coords scr_vec3 The coordinates where the pedestrian will spawn.
---@param heading number The heading of the pedestrian.
---@param register_as_network_object boolean? If true, registers the pedestrian as a network object.
---@param script_host_object boolean? If true, the pedestrian is hosted by the script.
---@return number The handle to the spawned pedestrian.
function spawn.ped(model, coords, heading, register_as_network_object, script_host_object) end

--- Spawns an object at the specified coordinates.
---@param model number|string The model hash or name of the object to spawn.
---@param coords scr_vec3 The coordinates where the object will spawn.
---@param register_as_network_object boolean? If true, registers the object as a network object.
---@param script_host_object boolean? If true, the object is hosted by the script.
---@param force_to_be_object boolean? If true, forces the entity to be an object.
---@return number The handle to the spawned object.
function spawn.object(model, coords, register_as_network_object, script_host_object, force_to_be_object) end

--- Spawns a pickup at the specified coordinates.
---@param model number|string The model hash or name of the pickup to spawn.
---@param coords scr_vec3 The coordinates where the pickup will spawn.
---@param flags number The flags associated with the pickup spawn.
---@param amount number The amount or value of the pickup.
---@param create_as_script_object boolean? If true, creates the pickup as a script object.
---@param script_host_object boolean? If true, the pickup is hosted by the script.
---@return number The handle to the spawned pickup.
function spawn.pickup(model, coords, flags, amount, create_as_script_object, script_host_object) end


util = {}

--- Returns the number of milliseconds since the system was started.
---@return number The current tick count
function util.get_tick_count() end

--- Splits a string into an array of strings based on the provided delimiter.
---@param str string The string to split.
---@param delimiter string The delimiter to use for splitting the string.
---@return string[] An array of strings obtained by splitting the input string.
function util.str_split(str, delimiter) end

--- Generates a random string of the specified length.
---@param length number The length of the random string to generate.
---@return string The generated random string.
function util.str_random(length) end

--- Converts a 4 byte integer into a list of bytes.
---@param number number The 4 byte integer to convert.
---@return number[] The list of bytes representing the integer.
function util.number_to_bytes(number) end

--- Converts a string of hex values to a list of bytes.
---@param str string The string of hex values to convert.
---@return string The list of bytes.
function util.str_to_bytes(str) end

--- Yields the current thread for a specified amount of milliseconds.
---@param ms number The number of milliseconds to yield the thread.
function util.yield(ms) end

--- Creates a tick thread that can execute natives and returns a handle for the thread.
---@param callback function The function to be executed in the thread.
---@return number The handle of the created thread.
function util.create_thread(callback) end

--- Removes a tick handler using its handle.
---@param id number The handle of the thread to remove.
function util.remove_thread(id) end

--- Generates a random integer
--- @param min number @The minimum value
--- @param max number @The maximum value
--- @return number
function util.random_int(min, max) end

--- Generates a random float
--- @param min number @The minimum value
--- @param max number @The maximum value
--- @return number
function util.random_float(min, max) end



-- Enums
---@enum GameState
GameState = {
    NONE = 0,
    INTRO_MOVIE = 1,
    LEGAL_SPLASH = 2,
    LEGAL_MAIN = 3,
    LANDING = 4,
    LOAD_GAME = 5,
    INSTALL = 6,
    LOAD_LEVEL = 7,
    MAP_CHANGE = 8,
}
---@enum OptClassId
OptClassId = {
    OPTION = 0,
    SKELETON = 1,
}
---@enum OptFlag
OptFlag = {
    IN_GAME = 0,
    SCRIPT_HOST_ONLY = 1,
    SESSION_HOST_ONLY = 2,
}
---@enum OptType
OptType = {
    BUTTON = 0,
    TOGGLE = 1,
    COLOR = 2,
    COLOR_LIST = 3,
    NUMBER_INT = 4,
    NUMBER_FLOAT = 5,
    COMBO = 6,
    TEXTBOX = 7,
    BREAKER = 8,
}
---@enum OptSubType
OptSubType = {
    INT = 0,
    FLOAT = 1,
    STRING = 2,
    BOOL = 3,
    VEC2 = 4,
    VEC3 = 5,
    SCR_VEC3 = 6,
    COLOR = 7,
}
---@enum OptScroll
OptScroll = {
    SCROLL = 0,
    TOGGLE = 1,
    PRESS = 2,
}
---@enum Directory
Directory = {
    CHEAT = 0,
    GAME = 1,
    IMAGES = 2,
    THEMES = 3,
    CONFIGS = 4,
    TRANSLATIONS = 5,
    SCRIPTS = 6,
    FONTS = 7,
    DLC = 8,
    SHV = 9,
    COMMANDS = 10,
    COLOR_PALETTES = 11,
    NETWORK = 12,
    WEAPON_LOADOUTS = 13,
    NETWORK_PROFILES = 14,
    NETWORK_HEADSHOTS = 15,
    OUTFITS = 16,
    VEHICLES = 17,
    MAIN_CONFIG = 18,
    LOG = 19,
    CRASH_LOG = 20,
    TEXTURE_SWAPS = 21,
    TELEPORT = 22,
    COLORS = 23,
    MAIN_HOTKEYS = 24,
    NETWORK_SEARCH_HISTORY = 25,
    NETWORK_PLAYER_HISTORY = 26,
    NETWORK_CHAT_SPAM = 27,
    NETWORK_RATE_LIMITS = 28,
    NETWORK_TRACKED = 29,
    NETWORK_ARTIFICIAL_ACQUAINTANCES = 30,
    NETWORK_BLACKLISTED_WORDS = 31,
    NETWORK_MODDER_DATABASE = 32,
    NETWORK_ADVERTISE = 33,
    NETWORK_TRANSACTIONS = 34,
    NETWORK_USER_BLOCKS = 35,
}
---@enum Notify
Notify = {
    INFO = 0,
    MODDER = 1,
    PLAYER_JOIN = 2,
    PLAYER_LEAVE = 3,
    PROTECTIONS = 4,
    SPAWNER = 5,
    HOTKEY = 6,
    WARNING = 7,
    SPOOFING = 8,
}
---@enum DataNodes
DataNodes = {
    AUTOMOBILE_CREATION = 0,
    BIKE_GAME_STATE = 1,
    BOAT_GAME_STATE = 2,
    DOOR_CREATION = 3,
    DOOR_MOVEMENT = 4,
    DOOR_SCRIPT_GAME_STATE = 5,
    DOOR_SCRIPT_INFO = 6,
    DYNAMIC_ENTITY_GAME_STATE = 7,
    ENTITY_ORIENTATION = 8,
    ENTITY_SCRIPT_GAME_STATE = 9,
    ENTITY_SCRIPT_INFO = 10,
    GLOBAL_FLAGS = 11,
    HELI_CONTROL = 12,
    HELI_HEALTH = 13,
    MIGRATION = 14,
    OBJECT_CREATION = 15,
    OBJECT_GAME_STATE = 16,
    OBJECT_ORIENTATION = 17,
    OBJECT_SCRIPT_GAME_STATE = 18,
    OBJECT_SECTOR_POS = 19,
    PATROL = 20,
    PED_AIDATA = 21,
    PED_APPEARANCE = 22,
    PED_ATTACH = 23,
    PED_COMPONENT_RESERVATION = 24,
    PED_CREATION = 25,
    PED_GAME_STATE = 26,
    PED_HEALTH = 27,
    PED_INVENTORY = 28,
    PED_MOVEMENT = 29,
    PED_MOVEMENT_GROUP = 30,
    PED_ORIENTATION = 31,
    PED_SCRIPT_CREATION = 32,
    PED_SCRIPT_GAME_STATE = 33,
    PED_SECTOR_POS_MAP = 34,
    PED_SECTOR_POS_NAV_MESH = 35,
    PED_TASK_SEQUENCE = 36,
    PED_TASK_SPECIFIC = 37,
    PED_TASK_TREE = 38,
    PHYSICAL_ANG_VELOCITY = 39,
    PHYSICAL_ATTACH = 40,
    PHYSICAL_GAME_STATE = 41,
    PHYSICAL_HEALTH = 42,
    PHYSICAL_MIGRATION = 43,
    PHYSICAL_SCRIPT_GAME_STATE = 44,
    PHYSICAL_SCRIPT_MIGRATION = 45,
    PHYSICAL_VELOCITY = 46,
    PICKUP_CREATION = 47,
    PICKUP_PLACEMENT_CREATION = 48,
    PICKUP_PLACEMENT_STATE = 49,
    PICKUP_SCRIPT_GAME_STATE = 50,
    PICKUP_SECTOR_POS = 51,
    PLANE_CONTROL = 52,
    PLANE_GAME_STATE = 53,
    PLAYER_AMBIENT_MODEL_STREAMING = 54,
    PLAYER_APPEARANCE = 55,
    PLAYER_CAMERA = 56,
    PLAYER_CREATION = 57,
    PLAYER_EXTENDED_GAME_STATE = 58,
    PLAYER_GAME_STATE = 59,
    PLAYER_GAMER = 60,
    PLAYER_PED_GROUP = 61,
    PLAYER_SECTOR_POS = 62,
    PLAYER_WANTED_AND_LOS = 63,
    SECTOR = 64,
    SECTOR_POSITION = 65,
    SUBMARINE_CONTROL = 66,
    SUBMARINE_GAME_STATE = 67,
    TRAIN_GAME_STATE = 68,
    VEHICLE_ANG_VELOCITY = 69,
    VEHICLE_APPEARANCE = 70,
    VEHICLE_COMPONENT_RESERVATION = 71,
    VEHICLE_CONTROL = 72,
    VEHICLE_CREATION = 73,
    VEHICLE_DAMAGE_STATUS = 74,
    VEHICLE_GADGET = 75,
    VEHICLE_GAME_STATE = 76,
    VEHICLE_HEALTH = 77,
    VEHICLE_PROXIMITY_MIGRATION = 78,
    VEHICLE_SCRIPT_GAME_STATE = 79,
    VEHICLE_STEERING = 80,
    VEHICLE_TASK = 81,
}
---@enum Justify
Justify = {
    LEFT = 0,
    RIGHT = 1,
    CENTER = 2,
}
---@enum Rounding
Rounding = {
    TOP_LEFT = 0,
    TOP_RIGHT = 1,
    BOTTOM_LEFT = 2,
    BOTTOM_RIGHT = 3,
}
---@enum NetBlock
NetBlock = {
    SYNC = 0,
    CREATE = 1,
    SELF_SYNC = 2,
}
---@enum NetObj
NetObj = {
    AUTOMOBILE = 0,
    BIKE = 1,
    BOAT = 2,
    DOOR = 3,
    HELI = 4,
    OBJECT = 5,
    PED = 6,
    PICKUP = 7,
    PICKUP_PLACEMENT = 8,
    PLANE = 9,
    SUBMARINE = 10,
    PLAYER = 11,
    TRAILER = 12,
    TRAIN = 13,
}
---@enum Event
Event = {
    PLAYER_JOINED = 0,
    PLAYER_LEFT = 1,
    PLAYER_FLAGGED = 2,
    CHAT_MSG = 3,
    CRASH_ATTEMPT = 4,
    SCROLL_WHEEL = 5,
    KEY_PRESS = 6,
}
---@enum NumberType
NumberType = {
    INT = 0,
    FLOAT = 1,
}
---@enum ComboType
ComboType = {
    INT = 0,
    FLOAT = 1,
    UINT64 = 2,
    STRING = 3,
    VEC2 = 4,
    VEC3 = 5,
    SCR_VEC3 = 6,
    COLOR = 7,
    BOOL = 8,
}
